var kbar, i, c, R, omegabar, Rk, n, sigma, spread, credit, bankrupt, GDP, wedge;

varexo sigma_e;

parameters sigma_ss mu gamma alpha delta beta rhosigma gam;

%dbstop in bgg_rbc_steadystate at 10
sigma_ss=0.26;
mu=0.21;
gamma=0.97;
alpha=1/3;
delta=0.02;
beta=1.03^(-.25);
rhosigma=0.97;
gam=1;

model;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Auxiliary expressions.  These simplify the equations without adding
% additional variables.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # F          = (normcdf(((log(omegabar) + sigma(-1)^2 / 2) / sigma(-1))));
  # G          = (normcdf((log(omegabar) + sigma(-1)^2 / 2) / sigma(-1) - sigma(-1)));
  # d          =  mu * G  * (1 + Rk) * kbar(-1);
  # Fp1        = (normcdf((log(omegabar(+1)) + sigma^2 / 2) / sigma));
  # Gp1        = (normcdf((log(omegabar(+1)) + sigma^2 / 2) / sigma - sigma));
  # GAMMA      = omegabar*(1-F)+G;
  # GAMMAp1    = omegabar(+1)*(1-Fp1)+Gp1;
  # dFp1       = normpdf((log(omegabar(+1)) + sigma^2 / 2) / sigma) / omegabar(+1) / sigma;
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Equilibrium equations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % resource constraint
    kbar(-1)^alpha = c + i + d ;
  
  % Law of motion for capital
    kbar = (1 - delta) * kbar(-1) + i;
  
  % Household FOC w.r.t. risk-free bonds
    1 = beta * (c(+1)/c)^(-gam)*(1+R);
  
  % efficiency condition associated with the contract
    (1-Fp1)/(1-GAMMAp1)-((1 + Rk(+1)) / (1 + R))*(1-Fp1-mu*omegabar(+1)*dFp1)/(1-((1 + Rk(+1)) / (1 + R))*(GAMMAp1-mu*Gp1));
  
  % Definition of return of entrepreneurs, Rk
    1 + Rk = alpha*kbar(-1)^(alpha-1)  + 1 - delta;
    
  % Zero profit condition
    kbar(-1)*(1 - ((1+Rk)/(1+R(-1)))*(GAMMA-mu*G)) - n(-1);
  
  % Law of motion of net worth
    n = gamma*(1-GAMMA)*(1+Rk)*kbar(-1);

  % borrowing:
    credit = kbar-n;

  % interest rate spread
    spread = ((1+Rk)/(1+R(-1)))*(kbar(-1)/credit(-1))*omegabar;

   % Shock 

   log(sigma / sigma_ss) = rhosigma * log(sigma(-1) / sigma_ss)  + sigma_e ;

   GDP = c + i;

   wedge = (1-(1+R)/(1+Rk(+1)));

   bankrupt = F;
end;

shocks;
var sigma_e; 
stderr .1;
end;


steady;

stoch_simul(order=1,irf=20,nograph) kbar, i, c, R, omegabar, Rk, n, sigma, spread, credit, bankrupt, GDP, wedge;
