function [varcoeff,Omega_VAR,cnst_VAR] = Yule_Walker(Ck,alf,p,YWcheck,tol_err)
%solve Yule-Walker equations and carries out a check on the solution

Nvar = size(Ck,1); 

YuleWalkerX = zeros(p*Nvar,p*Nvar);
YuleWalkerd = zeros(Nvar,p*Nvar);

for kkk = 0:p-1
    for jjj = 0:p-1
        firstcol = Nvar*kkk+1;
        lastcol = Nvar*kkk+Nvar;        
        firstrow = Nvar*jjj+1;
        lastrow = Nvar*jjj+Nvar;
        if jjj > kkk
            YuleWalkerX(firstrow:lastrow,firstcol:lastcol) = Ck(:,:,abs(jjj-kkk)+1)';
        else
            YuleWalkerX(firstrow:lastrow,firstcol:lastcol) = Ck(:,:,abs(jjj-kkk)+1);
        end        
    end
    
    YuleWalkerd(:,firstcol:lastcol) = Ck(:,:,kkk+2);
end

varcoeff = YuleWalkerd/(YuleWalkerX);

%check the solution is correct
if YWcheck == 1
    
    C_tau1check = zeros(Nvar,Nvar);
    C_tau1diff = zeros(Nvar,Nvar);
    for lll = 0:p-1
        if lll>0
            C_tau1check = C_tau1check + varcoeff(:,Nvar*lll+1:Nvar*(lll+1))*Ck(:,:,lll+1)';
        else 
            C_tau1check = C_tau1check + varcoeff(:,Nvar*lll+1:Nvar*(lll+1))*Ck(:,:,lll+1);
        end
    end
    C_tau1diff = C_tau1check - Ck(:,:,2);
    
    
    %check second of the Yule Walker equations
    C_tau2check = zeros(Nvar,Nvar);
    C_tau2diff = zeros(Nvar,Nvar);
    for lll = 0:p-1
        if lll>1
            C_tau2check = C_tau2check + varcoeff(:,Nvar*lll+1:Nvar*(lll+1))*Ck(:,:,abs(lll-1)+1)';
        else 
            C_tau2check = C_tau2check + varcoeff(:,Nvar*lll+1:Nvar*(lll+1))*Ck(:,:,abs(lll-1)+1);
        end
    end
    C_tau2diff=C_tau2check - Ck(:,:,3);
    
    %check last of the Yule Walker equations
    C_taupcheck = zeros(Nvar,Nvar);
    C_taupdiff = zeros(Nvar,Nvar);
    for lll = 0:p-1
        if lll>p-2
            C_taupcheck = C_taupcheck + varcoeff(:,Nvar*lll+1:Nvar*(lll+1))*Ck(:,:,abs(lll-(p-1))+1)';
        else 
            C_taupcheck = C_taupcheck + varcoeff(:,Nvar*lll+1:Nvar*(lll+1))*Ck(:,:,abs(lll-(p-1))+1);
        end
    end
    C_taupdiff=C_taupcheck - Ck(:,:,p+1);
    
    YuleWalkerTol = max([max(max(abs(C_tau1diff))) max(max(abs(C_tau2diff))) max(max(abs(C_taupdiff)))]);
    if YuleWalkerTol > tol_err
        fprintf('Max abs error of %15.12f in YW eqns exceeds tolerance of 1.0e-10',YuleWalkerTol); 
    end
    
end

%compute the covariance matrix
Omega_VAR = Ck(:,:,1);
for ii = 1:p
    Omega_VAR = Omega_VAR - varcoeff(:,(ii-1)*Nvar+[1:Nvar])*Ck(:,:,ii+1)';
end

%compute the constant of the VAR
cnst_VAR = eye(size(varcoeff,1));
for lll = 1:p
    cnst_VAR = cnst_VAR - varcoeff(:,Nvar*(lll-1)+1:Nvar*lll);
end;
cnst_VAR = cnst_VAR*alf;
