function [Ck,Nvar_c,nameXX,max_eig_] = autocov_var(varcoeff,V,p,hasconstant,k,N,lambda_hp,index_trans,tauvar)
%compute autocovariances in time and/or frequency domain for a VAR(p):
%  Y(t) = c + A1*Y(t-1) + A2*Y(t-2) + ... +   Ap*Y(t-p) + u(t), 
%
%Inputs:
% varcoeff = [c A1 A2 ... Ap],
% V = Eu(t)u(t)'
% p = order of VAR
% hasconstant= 1, if constant was included in estimation. In this case, first column of varcoeff is the constant; =0, no constant (thus, first column is not the constant)
% k = order of autocovariance you want to compute
% N  = number of points used to evaluated the integral in frequency domain
% lambda_hp
% index_trans: a cell containing the list of variables for which you want to compute the autocovariances, and the transformations you want to apply (e.g. differencing, H-P filtering), and the string with name
% tauvar: this will normally be a vector [1:number-of-equation]

do_freq = 0; % compute autocovariance also in frequency domain. If you don't want to transform variables (e.g. differencing) and don't want HP filter, the program will compute autocovariances in the time domain only. If you set do_freq=1, it will do it also in the frequency domain. This is useful as a check on the computations

chkRieman = 0; %=1,some sort of check on the Riemann approximation to the integral
tol_err = 10^-10;


%--
%drop variables (listed in first column of index_trans) that are not
%included in tauvar, and re-number entries to reflect size of tauvar. Finally, construct a selection matrix JJ
JJ = zeros(1,size(tauvar,2));
index_do_trans = 0;
index_do_hp = 0;
ii = 1;
for jj = 1:size(index_trans,1)
    rob = index_trans{jj,1};
    ij__ = [];
    for ij = 1:size(rob,2)
      rob__ = tauvar==abs(rob(1,ij));
      if max(rob__) == 1
          ij__ = [ij__  sign(rob(1,ij))*find(rob__)];
      end
    end
    if length(ij__) == ij
        index_{ii,1} = ij__;
        index_(ii,2) = index_trans(jj,2);
        index_(ii,3) = index_trans(jj,3);
        index_(ii,4) = index_trans(jj,4);
        
        JJ(ii,abs(ij__)) = sign(ij__);
        ii = ii +1;
        
        index_do_trans = index_do_trans + sum(index_{jj,2});
        index_do_hp = index_do_hp + sum(index_{jj,3});
    end
end

%construct string with names
nameXX = index_{1,4};
for jj = 2:size(index_,1)
    nameXX = strvcat(nameXX,index_{jj,4});
end

%--start computations

if hasconstant==1
    A = varcoeff(:,2:end);
    const = varcoeff(:,1);
else
    A = varcoeff(:,1:end);
    const = zeros(size(varcoeff,1),1);
end

Nvar = size(varcoeff,1);
Nvar_c = size(index_,1);

%construct companion form
AA = zeros(Nvar*p,Nvar*p);
AA(1:Nvar,:) = A;
for ii = 1:p-1
    AA(ii*Nvar+[1:Nvar],(ii-1)*Nvar+[1:Nvar]) = eye(Nvar);
end

max_eig_ = max(abs(eig(AA))) ;
disp(['Biggest eigenvalue in VAR is ' num2str(max_eig_)])
if max_eig_ > 1
    error('Explosive Eigenvalue in VAR')
end


%--- (1)  Time domain
if index_do_trans ==0  && index_do_hp ==0

    V_comp = zeros(Nvar*p,Nvar*p);
    V_comp(1:Nvar,1:Nvar) = V;

    [Sigma_comp]  =   dare(AA',zeros(size(AA)),V_comp);
    if max(max(abs( Sigma_comp - AA*Sigma_comp*AA' - V_comp ))) > .1e-8
        error('fatal (autocov_var) failed to solve Riccati Equation')
    end

    Ck_t = zeros(Nvar,Nvar,k+1);
    Ck_t_JJ = zeros(Nvar_c,Nvar_c,k+1);

    Ck_t(:,:,1) = Sigma_comp(1:Nvar,1:Nvar);
    Ck_t_JJ(:,:,1) = JJ*Ck_t(:,:,1)*JJ';

    for jj = 1: min(p-1,k)
        Ck_t(:,:,jj+1) = Sigma_comp(1:Nvar,jj*Nvar+[1:Nvar]);
        Ck_t_JJ(:,:,jj+1) = JJ*Ck_t(:,:,jj+1)*JJ';
    end

    kk = p;
    while kk <= k
        Ck_kk = zeros(Nvar,Nvar);
        for jj =1:p
            Ck_kk = Ck_kk + A(:,(jj-1)*Nvar+[1:Nvar])*Ck_t(:,:,kk+1-jj);
        end
        Ck_t(:,:,kk+1) = Ck_kk;
        Ck_t_JJ(:,:,kk+1) = JJ*Ck_t(:,:,kk+1)*JJ';
        kk = kk+1;
    end

    Ck = Ck_t_JJ;
end

%--- (2) Frequency domain
if index_do_trans ~=0 || index_do_hp ~= 0 || do_freq==1

NN = N/2;

om = 2*pi*[(1:NN+1)-1]/N;

[g1,g2] = HPSPECT(lambda_hp);

zzz = exp(-sqrt(-1)*om);

fz = (-g2*(1-zzz).*(1-zzz).*(1-1./zzz).*(1-1./zzz)./((1-g1*zzz-g2*zzz.^2).*(1-g1./zzz-g2./zzz.^2)));
fztil = (-g2*(1-zzz).*(1-1./zzz).*(1-1./zzz)./((1-g1*zzz-g2*zzz.^2).*(1-g1./zzz-g2./zzz.^2)));

Spec = zeros(Nvar_c,Nvar_c,length(om));

for ii = 1:length(om)
    JJj = JJ;

    %transform variables as specified in 2nd and 3rd columns of index_
    for jkj = 1:size(index_,1)
        rob1 = index_{jkj,1};
        rob2 = index_{jkj,2};
        rob3 = index_{jkj,3};
        
        for iki = 1:size(rob1,2)

            if rob2(iki) ==0 && rob3(iki)==0
                ff_ = 1;
            elseif rob2(iki) == 2 && rob3(iki)==0
                ff_ = 1-zzz(ii);
            elseif rob2(iki) ==0 && rob3(iki)==1
                ff_ = fz(ii);
            elseif rob2(iki) ==2 && rob3(iki)==1
                error('you have chosen to transform a variable in first difference and to apply HP filter on that transformed variable. This makes little sense')
            elseif rob2(iki) ==1 && rob3(iki)==1
                ff_ = fztil(ii);
            elseif rob2(iki) ==1 && rob3(iki)==0
                error('you have chosen to transform a variable in level and do not want to apply HP filter on that transformed variable. This is not defined at omega=0')
            end

            JJj(jkj,rob1(iki)) = JJ(jkj,rob1(iki))*ff_;
        end
    end

    S_y_pp = eye(Nvar,Nvar);

    for pp = 1:p
        zz = exp(-sqrt(-1)*pp*om(ii));

        S_y_pp = S_y_pp - A(:,(pp-1)*Nvar+[1:Nvar])*zz;
    end

    S_y = inv( S_y_pp );

    Spec(:,:,ii) = JJj*S_y*V*S_y'*JJj';
end


%Compute the autocovariances C(0),C(1),...,C(k) exploting some properties (see technical appendix)
%Save in Ck_f(:,:,1)=C(0), ... Ck_f(:,:,k+1) = C(k)
Ck_f = zeros(Nvar_c,Nvar_c,k+1);

if max(max(abs(imag(Spec(:,:,1)))))
    error('fatal Spectral density at omega=0 is complex')
end

for kkk = 1:k+1
    
    Ck_f(:,:,kkk) = real(Spec(:,:,1)); %the imaginary part should be 0 in theory
    
    for jjj = 2:NN
        w = (pi*(jjj-1))/NN;
        Ck_f(:,:,kkk) = Ck_f(:,:,kkk) + 2*real(Spec(:,:,jjj)*exp(sqrt(-1)*w*(kkk-1)));
    end
    Ck_f(:,:,kkk) = (Ck_f(:,:,kkk) + real(Spec(:,:,NN+1)*exp(sqrt(-1)*pi*(kkk-1))))/(2*NN);
end

%the check below computes autocovariances integrating the spectrum between -pi and pi
%this provides a check on the computations above
if chkRieman == 1
    checkRieman(k,Nvar_c,Spec,Ck_f,NN,tol_err)
end

if index_do_trans == 0 && index_do_hp ==0
%check whether autocovariances computed in time and frequency domain are equal
if max(max(max(max(Ck - Ck_f)))) > tol_err
    error(['fatal autocovariances computed in time and frequency domain differ by more than ' num2str(tol_err)] )
end
end

Ck = Ck_f;
end



