%-------------------------------------------------------------------------
% This is the main program to transform a matlab .m file into a dynare .mod file
% to be used for estimation or for computing optimal policy. To compute
% optimal policy it makes use of A.Levin's code
%-------------------------------------------------------------------------

% Note: make sure that in the file (eg. nonlin.m) to be transformed, auxiliary variables do NOT have the same name used for endogenous variables as listed in the vector zp1,z0,zm1 and psip1,psi0

% Note: conversion to dynare handles cases up to 3-period leads and 3-period lags. You have to modify the file create_dynfile.m if your model has more than 3-period leads/lags

% Note: initial values for s.d. of measurement errors are taken from the
% file parametersstoch.m (vector Vmeas). Make sure they are the same as in
% vector xx containing the parameter value you like

% Exogenous variable processes are written as AR(1). If you want to modify it, change addshocks.m

if user == 1
    addpath c:\projects\ECB\August2005\create_dynfile
else
    addpath([curdrive 'create_dynfile']);
end

model_dynare = 1; %=1 if you are interested in doing estimation in dynare; =0 if you are interested in optimal policy in dynare 

filename = 'nonlinnew.m'; %specify name of the file containing non-linear equations

filename_obs = 'nonlin_obs.m'; %name of file containing non-linear equations that descibe mapping between model and data

name_file_prior  = 'dyn_par_prior.mat'; %name of file containing parameters to be estimated and priors

%specify the parameters that govern conditional execution in the file containing the nonlinear equations
ifstatements = {['pref=' num2str(pref)];... 
                ['entr=' num2str(entr)];...
                ['timing=' num2str(timing)];...
                ['rhobank=' num2str(rhobank)];...
                ['Taylor=' num2str(Taylor)];...
                ['country=' num2str(country)];...
                ['forward=' num2str(forward)];...
                };

change_timing = {{'kbar', '(-1)'}; ...  %specify the name of variables (without any suffix, e.g. kbarp1 becomes kbar) that need to be shifted in order to match dynare convention. The second column specifies the needed time shift
                 {'dm', '(-1)'}; ...     
                 {'mb', '(-1)'};...
                 {'n', '(-1)'};...
                 {'Rm', '(-1)'};...
                 {'Re', '(-1)'};...
                };
policyrule = {'1','ff\(23\)'}; %first string specifies whether the policy rule is included in the file to be processed (0 if not included, 1 if included). The second string specifies the equation number for the policy rule (e.g. 'ff\(23\)') (Note that backslash symbol (i.e. \) in front of the parenthesis is needed for the syntax of the program). If taylor rule is not included, leave the second string empty: ('') 
  
delete_eqs = {'ff\(38\)', 1}; %specify the equation-number related to the equations included in the file and that you do not want to include in the .mod version of the model, e.g. delete_eqs = {ff\(3\), 1; ff\(6\), 0}; Leave it empty if you want to include all the equations: {''}; The second column specifies whether you want to delete the equation in .mod file for estimation (set it to 1) or optimal policy (set it to 0)
                                    %you don't need to delete policy-rule equation when model_dynare=0. The program will do it automatically
suff_capletters = 'X'; %if name contains capital letter, the program adds 'X' at the end of the name (this is done because dynare is case insensitive)
suff_var = 'U';  %name of variables are modified by adding 'U' at the end
suff_param = 'UU'; %name of parameters are modified by adding 'UU' at the end

replace_var =  {'1', 'iota2', '1-iota1'; ...     %1th column specifies whether the replacement has to occur in model_dynare=0 and/or =1. If you want the replacement in both cases, write '0|1'; 2nd column, variable to be replaced; 3rd column, replacement variable. If you don't want any replacement, just specify first row as 3 empty strings
                '1', 'iotaw2', '1-iotaw1'; ...
                '0|1', 'xp0', ['e_xp' suff_var]}; 
                
%Specify household's period utility and welfare:
if entr >= 0
    util_index = {'Util', '= zetac0*(log(c0*muzstar0-b*cm1)-zeta0*psiL/(1+sigmaL)*(wstar0/wplus0)^((1+sigmaL)*lambdaw/(lambdaw-1))*h0^(1+sigmaL)-v0/(1-sigmaq)*((1+tauc0)*pi0*c0*muzstar0/(mb0*m0^(theta0*(1-chii0))*(1-m0)^((1-chii0)*(1-theta0))*dm0^chii0))^(1-sigmaq)-(exp(sqrt(Hdoupr/2)*(m0*(1+xm1)/mm1-pi*muzstar))+exp(-sqrt(Hdoupr/2)*(m0*(1+xm1)/mm1-pi*muzstar))-2))'};
else
    util_index = {'Util', '= zetac0*(log(c0*muzstar0-b*cm1)-zeta0*psiL/(1+sigmaL)*(wstar0/wplus0)^((1+sigmaL)*lambdaw/(lambdaw-1))*h0^(1+sigmaL))'};
end
welfare    = {'Welf', '= Util + nbeta*Welf(+1)'};

%Names of planner's discount factor used in Andy's code and of
%household's discount factor in the model to be converted to dynare
discount ={'nbeta','beta'}; %the two will be set the same by the programs below

%%-------------------------------------------------------------------------
% The execution of the program starts below. All options to be set are above
%--------------------------------------------------------------------------
[out_folder,name1,name2,name3,util_index] = transform2dynare(filename,ifstatements,model_dynare,replace_var, ...
                       name_file_prior,filename_obs,suff_param,suff_var,suff_capletters,util_index);  %it prepares the file for conversion to dynare

 
% err = check_transform(out_folder,filename,name3,name1,ifstatements);
% if err ~= 0
%     error('(fatal) transformations made by transform2dynare.m are in all likelihood wrong')
% end

[filename_mod,change_timing,util_index] = create_dynfile(out_folder,name3,name1, ...
                  change_timing,policyrule,model_dynare,delete_eqs,suff_capletters,suff_var,suff_param,util_index);  %it starts creating a file in dynare format and saves it as .mod 

set_dynparams1 = 'set_dynparams'; %name of file that will contain the parameters and ss computations ready for dynare

if model_dynare ==1 %if you are interested in doing estimation in dynare
    
    [filename_mod_nt] = ch_time_struct(out_folder,filename_mod,change_timing); %it changes the timing of some variables (e.g. capital) in order to meet dynare's convention

    [out_file_name] = addshocks(out_folder,filename_mod_nt,name1,suff_capletters,suff_var,suff_param,set_dynparams1);
         
    [out_file_name] = param_prior_dyn(out_folder,out_file_name,name_file_prior,suff_capletters,suff_var,suff_param,name1);

    [set_dynparams1] = file_param_dynare(out_folder,name1,set_dynparams1,model_dynare,suff_param,suff_capletters,suff_var,name_file_prior);
    
    [out_file_name] = last_fix1(out_folder,suff_var,suff_param,out_file_name); %this file makes some ad hoc changes to fix idiosyncratic problems
    last_fix2(out_folder,suff_var,suff_param,set_dynparams1); 
    
    curdir=which('main_mat2dyn');
    curdir=fileparts(curdir);
    cd([curdir out_folder]);
    set_dynparams
    [out_file_name] = setinit_estim(out_folder,out_file_name,name1);
    cd('..');
    
elseif model_dynare ==0 %if you want to do optimal policy in dynare using A.Levin's code

    [namefile_nt] = prepare_optmod(out_folder,filename_mod,change_timing,util_index,welfare,discount); %it also calls the function that shifts the timing of variables
    
    [namefile_nt] = addshocks(out_folder,namefile_nt,name1,suff_capletters,suff_var,suff_param,set_dynparams1);

    curdir=which('main_mat2dyn');
    curdir=fileparts(curdir);
    
    infilename = namefile_nt(1:end-4);
    outfilename=[namefile_nt(1:end-4) '_op'];
    
    cd([curdir out_folder]);
    addpath([curdrive 'Get_Ramsey']);
    get_ramsey; %call get_ramsey
    rmpath([curdrive 'Get_Ramsey']);
    cd('..');
    
    [outfilename] = adjust(out_folder,[outfilename '.mod'],suff_capletters,suff_var,suff_param,discount,util_index,welfare); 

    [set_dynparams1] = file_param_dynare(out_folder,name1,set_dynparams1,model_dynare,suff_param,suff_capletters,suff_var,name_file_prior);
    
    last_fix2(out_folder,suff_var,suff_param,set_dynparams1); %make some adjustments to set_dynparams1
    
    [mod_nonoptim] = last_fix1(out_folder,suff_var,suff_param,namefile_nt); %make adjustments to mod file with Taylor rule (i.e. non optimal policy)
    [mod_nonoptim] = adjust(out_folder,mod_nonoptim,suff_capletters,suff_var,suff_param,discount,util_index,welfare); %make adjustments to mod file with Taylor rule (i.e. non optimal policy)
    
    curdir=which('main_mat2dyn');
    curdir=fileparts(curdir);
    cd([curdir out_folder]);
    set_dynparams   
    %cd('..');
end
rmpath([curdrive 'create_dynfile']);