%------------------------  THIS IS THE MAIN PROGRAM  -------------------------------
clear all; close all
dbstop in micro at 4
%dbstop in makealphabeta at 135

%--------------------------------------------------------------------------
%       Following are parameters that user has to set to run this program
%--------------------------------------------------------------------------
user = 1; % =0, ECB users; =1, Larry. Go to file setpath.m to make sure the folders' paths are right
namedir = 'euroarea_code'; %specify folder's name containing the main code
addpath('C:\projects\ECB\August2005\euroarea_code_6_10_2009\euroarea_code\data')
addpath('C:\projects\ECB\August2005\euroarea_code_6_10_2009\euroarea_code')
irun = 1; % =0, compute posterior mode via maximum likelihood and priors
% =1, compute and plot impulse responses
% =2, get model's VAR
% =3, run non-Bayesian VAR using the same observables as in the DSGE model and plot forecasts
% =4, do forecast using the DSGE model using the Kalman filter
% =5, simulations showing role played by different shocks in the dynamics of Kalman Smoothed Variables
% =6, run BVARs and simple benchmarks
% =7, construct Dynare version of the model
% =8, compute second moment properties for HP-filtered versions of a subset of the observed variables.


% parameter values must be set in 'parameterseuro' for euroarea and 'parametersus59_massi' for US. Parameters related to stochastic processes are in 'parametersstoch_ea' and 'parametersstoch_us'

country = 2; % =1, euroarea; =2, US (starting in 1983); =3, US (starting in 1959)

taushock2=[]; %shocks that you want to follow AR(2) representation. If empty, it is assumed you do not want AR(2) terms

doRBC =0; %=0 don't compute RBC version version of the model and don't do plots that display alternative models
%=1 if you want to compute RBC version of the model and plot impulse responses of the model and RBC on the same chart
%=2 do what doRBC = 1 does, plus do a single graph with just the RBC model responses and save those responses to a local file
%=3 do what doRBC = 1 does, plus do a graph of the RBC model responses and the responses saved in doRBC=2
%=4 store information on CEE model (entr=-2) for later use in plotting
%=5 use data generated under doRBC=4 as baseline in current plot (with =5, you must have run with =4 first)
%==========================================================================
%-------  Options related to estimation, forecasts and VAR/BVAR
%--------------------------------------------------------------------------

%--->parameters to be estimated, their bounds and priors must be specified in set_params_priors.m

%-------- Select SHOCKS you want to activate by setting them to zero or one
qq(1)=1;%1 lambdaf
qq(2)=1;%2 inflation objective
qq(3)=0;%3 fraction of wage bill to be financed
qq(4)=0;%4 fraction of capital bill to be financed
qq(5)=1;%5 ksi ****
qq(6)=1;%6 technology shock in banking sector *****
qq(7)=1;%7 investment specific shock
qq(8)=0;%8 theta
qq(9)=1;%9 chi *****
qq(10)=0;%10 tax rate on labor
qq(11)=0;%11 capital income tax rate
qq(12)=0;%12 zeta
qq(13)=1;%13 g
qq(14)=0;%14 v
qq(15)=0;%15 tauD
qq(16)=1;%16 muz
qq(17)=1;%17 gamma *****
qq(18)=1;%18 epsilon
qq(19)=1;%19 monetary policy shock
qq(20)=0;%20 tax on consumption
qq(21)=1;%21 sigma *****
qq(22)=1; %22 consumption shock
qq(23)=1; %23 shock to the maginal efficiency of investment
qq(24)=1; %24 shock to the cost of capital utilisation
qq(25)=1; %25 shock to term structure *****

%--------- Select VARIABLES you want to include in the estimation by setting them to zero or one
zz(1)=1;%1 real net worth growth (or, divided by gnp if fdiff = 0) *****
zz(2)=1;%2 inflation
zz(3)=1;%3 log hours
zz(4)=1;%4 credit growth ****
zz(5)=1;%5 output growth
zz(6)=1;%6 real wage growth (or, divided by gnp)
zz(7)=1;%7 investment growth (or, divided by gnp)
zz(8)=1;%8 real M1 growth (or, log m1 velocity) *****
zz(9)=1;%9 real M3 growth (or, log broader m velocity) *****
zz(10)=1;%10 consumption growth (or, divided by gnp)
zz(11)=1;%11 risk premium          *****
zz(12)=1;%12 Spread1 (Rlong-Re) *****
zz(13)=0;%13 Ra
zz(14)=1;%14 Re
zz(15)=0;%15 capacity utilization
zz(16)=0;%16 government consumption growth
zz(17)=1;%17 relative price of investment
zz(18)=1;%18 real oil price growth
zz(19)=0;%19 Spread2 (Medium-Re)
zz(20)=1;%20 Banks' Reserves ****

start_estimation = 1981.0; %1981.0
end_estimation   = 2008.25; %2008.25
end_estimation_means = end_estimation;
init_kalman      = 16; %16; %number of periods used to initialise the filter; =0 no initialisation
demean           = 1; %=1 if you want to demean all data; =0 only some variables are demeaned (the ones NOT listed in mm below); mean is computed over the period we evaluate the likelihood up to end_estimation
mm = [1 2 4:14 16:18 20]; %variables whose mean should be equated to data mean (variables NOT listed here will be demeaned also if demean=0); if demean=1, mm is not used

if country ==3
    start_estimation = 1959.0;
end

tauvar2   = zeros(size(find(zz==1))); %variables for which you want to estimate measurement error: =1 if you want to estimate it; =0 if you don't want to estimate it (in this case the program reads the value for measurement error specified in parametersstoch_ea and _us)

taushock1 = [ 2  19]; %shocks for which you do NOT want to estimate the AR(1) coefficient; if a shock is included in this list, the program reads the value of ar(1) coefficient specified in parametersstoch.m. Leave it empty if you want to estimate all AR(1) terms
taushock3 = [2]; %shocks' standard deviations you do NOT want to estimate; if a shock is included in this list, the program reads the value of the std specified in parametersstoch. Note that if you activate the signal on a shock that is included in this list, the program assumes that you do not want to estimate the std of the signal as well

shock_trans = [3 4 8 10 11 15 20]; %set the shocks you want to be transformed in order to constrain them to lie in unit interval
%[3 4 5 8 9 10 11 15 17 20];

fdiff   = 1; % =1, variables are estimated in first difference (e.g. Dy, Dc(t), Di(t), Dw(t), Dn(t+1), Dm3(t), but L, R, Dp; =0, variables are estimated in ratios: c/y, i/y, w/y, n/y, velocity of m3 and m1, but L, R, Dp, Dy;
detrending = 0; % =1, the program lenearly detrends data before estimation; =2, it does HP filter; =0, it does not detrend anything; =3, it linearly detrends M1 and M3 velocity only

ric     = 0; %  =1, uses the Kalman steady state matrix P as P_{1|0}
% ~=1 does 100 iterations from an arbitrary initial condition, on Riccati equation

opt_routine = 4; %choose optimisation routine to compute the posterior mode
% =0, do not perform any optimisation, just use the calibrated values or values previously estimated
% =1, Matlab's fminsearch
% =2, Matlab's fminunc
% =3, Matlab's fminunc, supplying the Gradient and Hessian
% =4, SolvOpt (gradient-based method with space dilation)
% =5, Sims' routine modified to introduce bounds to parameters (NOTE: you need to go inside 'maximize.m' and set the bounds)
% =6, standard simulated annealing by Goffe (NOTE: you need to go inside 'maximize.m' and set the bounds)
% =7, ASA (adaptive simulated annealing). (NOTE: you need to go inside 'maximize.m' and set the bounds)

%=========================================================================
%------ Alternative specifications of the model
%-------------------------------------------------------------------------
entr    = 1;  %=1, benchmark model (with entrepreneur and banks); =0, benchmark without entrepreneurs; =-1, benchmark with no entrepreneurs or banks
%=-2, model without entrepreneurs, banks or money,  and psil=psik=0
%=2, benchmark model with REAL state-noncontingent returns to household
%=-3, BGG, i.e. model with entrepreneurs and no banks or money, and psil=psik=0
%=-4, BGG model with REAL state-noncontingent returns to household

if entr ==1 || entr==2
    if country==1

        qq = [1 1 0 0 0 1 1 0 1 0 0 0 1 0 0 1 1 1 1 0 1 1 1 1 1]; %
        zz = [1 1 1 1 1 1 1 1 1 1 1 1 0 1 0 0 1 1 0 0]; %
        taushock1 = [2 19]; %taushock1 = [2 17 19];  %shocks for which you do NOT want to estimate the AR(1) coefficient; if a shock is included in this list, the program reads the value of ar(1) coefficient specified in parametersstoch.m. Leave it empty if you want to estimate all AR(1) terms
        taushock3 = [ 2]; %shocks' standard deviations you do NOT want to estimate; if a shock is included in this list, the program reads the value of the std specified in parametersstoch. Note that if you activate the signal on a shock that is included in this list, the program assumes that you do not want to estimate the std of the signal as well
        tauvar2   = [1 0 0 1 0 0 0 1 1 0 1 1 1 0 0]; %EA-CMR

        detrending=3; %0

    elseif country ==2
        qq = [1 1 0 0 1 1 1 0 1 0 0 0 1 0 0 1 1 1 1 0 1 1 1 1 1];
        zz = [1 1 1 1 1 1 1 1 1 1 1 1 0 1 0 0 1 1 0 1];
        tauvar2 = [1 0 0 1 0 0 0 1 1 0 1 1 1 0 0 1];%US-CMR
    end
elseif entr ==-3 ||entr==-4
    tauvar2  = [1 0 0 1 0 0 0 0 1 1 0 0];%BGG
    qq = [1 1 0 0 0 0 1 0 0 0 0 0 1 0 0 1 1 1 1 0 1 1 1 1 0];
    zz = [1 1 1 1 1 1 1 0 0 1 1 0 0 1 0 0 1 1 0 0];
elseif entr==-2
    qq = [1 1 0 0 0 0 1 0 0 0 0 0 1 0 0 1 0 1 1 0 0 1 1 1 0];
    zz = [0 1 1 0 1 1 1 0 0 1 0 0 0 1 0 0 1 1 0 0];
end

Taylor  =   2;% specify the monetary policy rule; =0, money growth rule;
% =0, exogenous money base rule;
% =1, Taylor rule on Rb
% =2, Taylor rule on Re
% =3, money base rule with feedback

forward  = 1;  % =1, the inflation term in the Taylor rule is 1 period forecast; =0, it is contemporaneous; =-1, it is 1 period lag
begin    = 0; %=1, M1, M3, Credit are defined in terms of beginning of period quantities; =0 in terms of end of period
calvohours = 0; %=0 use data on hours worked; =1 use employment data and add a calvo equation linking employment and hours

pref    = 1; % =1, benchmark preferences, =0, ACEL preferences
costreal= 1; % =0, adjustment cost in changing money holding is in terms of nominal money growth; =1, the cost is in terms of real money growth
util    = 1; % =1, variable capital utilisation; =0 remove variable capital utilisation
static  = 0; % =0, dynamic adjustment cost in investment; =1, static adjustment cost in investment

money   = 3;  %Information restrictions:  =3, you see everything
% =2, you see everything but the policy shock
% =0, you only see technology shocks
% =1, except for monetary policy shock, you don't see financial market shocks...money you do see. (works for entr=1)

rpl     = 1; %if you want Leontief production function for bank deposits, set rp1=0 here and rhobank<0 in parameterseuro

timing   =1; % =1, timing is set according to equation (22) in the Manuscript; =0, timing is set according to (22)'

do_qavrg =0; %=1, compute also average q; =0, do not compute it (note: computing q average worsens the degree of accuracy of solution. so we don't compute it unless it is needed)

% information restrictions are specified by taumat in file 'makealphabeta' and by taup and part in 'checkB.m'

%=========================================================================
%------- Options related to checks
%-------------------------------------------------------------------------
icheck = 0; % =1, (NOTE: it works when only when irun=1) check the analytic derivatives contained in the alpha's and beta's by comparing them with derivatives obtained numerically

docheck_XXX = 0; %=1, perform the checks on the tau's and on alf (i.e. the mapping from observable variables to model variables) by comparing analytical and numerical derivatives

accuracy  = 6.4e-7;  %lowest accetable degree of accuracy in solving for the feedback part

%-------------------------------------------------------------------------
% End of options setup. Below the programs start
%-------------------------------------------------------------------------
if entr == -1
    error('fatal (main) this option has not been fully debugged')
end

setpath

tauvar    = find(zz==1);%index of variables that you have chosen to include in the estimation
taushock  = find(qq==1);%shocks that you want to activate

if irun==1
    disp('')
    disp('...type in at the prompt:')
    disp('[i,j,...];')
    disp('where i,j,... are the number(s) of the shocks for which you want')
    disp('to plot impulse responses, then press the return key')
    taushock=input('');
    %taushock= [21];

    taushock1=[];
    taushock3=[];
end

[taushock1,taushock3] = checknumshocks(taushock,taushock1,taushock3);

nn = 0;
if do_qavrg==1
    nn = 1;
end

if entr == 1 || entr == 2
    nn= nn+30; %30 %number of variables included in z(t). See Manuscript
elseif entr == 0
    nn= nn+27;
elseif  entr == -1;
    nn= nn+18;
elseif entr == -2;
    nn= nn+14;
elseif entr == -3 || entr==-4
    nn= nn+18;
end


nn_shocks = length(qq); %total number of shocks potentially available in the model

%get model parameters
if country==1 %euroarea
    if entr==1
        parameterseuro
    elseif entr==-3
        parameterseuro_bgg
    elseif entr==-2
        parameterseuro_cee
    elseif entr ==2
        parameterseuro_nofisher
    end
    parametersstoch_ea %parameters that pertain to the time series representation of the exogenous stochastic processes
elseif country==2 %US (starting in 1982)
    if entr==1
        parametersus59_massi
    elseif entr==-3
        parametersus59_massi_bgg
    elseif entr==-2
        parametersus59_massi_cee
    elseif entr==2
        parametersus59_massi_nofisher
    end
    parametersstoch_us
elseif country==3 %US (starting in 1959)
    error('fatal (main) you are using country=3. You should verify that the steady state is computed correctly and the values make sense')
    parametersus59
end

loadparameters;

if irun==1
    load_estimates = 1;
    if load_estimates ==1
        if country == 1
            if entr == -2
                load results_cee_ea xx par
            elseif entr == 1
                load results_cmr_ea xx par
            elseif entr == -3
                load results_bgg_ea xx par
            elseif entr ==2
                load results_cmr_Fisher_ea xx par
            end
        elseif country == 2
            if entr == -3
                load results_bgg_us xx par
            elseif entr == 1
                load results_cmr_us xx par
            elseif entr==-2
                load results_cee_us xx par
            end
        end
        undo; 
        %xiw=.01;
        loadparameters;
    end
end

%solve for the steady state
[pi,s,rk,i,u,omegabar,Rk,n,q,vl,ev,mb,R,uzc,lambdaz,m,Ra,c,w,l,k,kbar,Re,dm,Rm,m3,btot,pstar,wstar, ...
    Fp,Fw,Yz,wplus, ...
    g,y,phi,RT,m1,Rb,Pe,currdep,costs,vk,pitarget, ...
    psiL,theta,chii,tau, ...
    xb,ksi,psik,psil,chorse,hkb,her,erz, ...
    sigma_SS,vn,bigtheta,mu,we,Fomegabar,pibar,params] = steadystate(rk,params);

micro(sigma_SS,omegabar,Rk,Re,k,n,Pe,mu);
% [edf] = financial(sigma_SS,omegabar,Rk,Re,k,n,Pe,mu);

name_table=[outfolder '\eurotable.tex'];

%eurotable87: for US it displays values computed over 1987-2003
eurotable(phi,beta,psiL,sigmaL,v,sigmaq,theta,Hdoupr,b,xiw,lambdaw,muzstar,Sdoupr,sigmaa,xip,psik,psil,delta,alpha, ...
    lambdaf,gamma,mu,Fomegabar,omegabar,sigma,ksi,xb,tau,tauc,tauk,taul,x,k,y,i,c,g,rk,n,we,Rk,vl,pi,m,mb,w,l, ...
    her,Re,R,Ra,RT,Rm,Rb,dm,chii,chorse,bigtheta,upsil,m3,m1,currdep,vn,name_table,entr,params);

if (abs(rhobank) > .1e-9) && (rpl ~= 1)
    disp('fatal (main) wrong value for rhobank. the steady state code has been adjusted')
    disp('for a nonzero value of rhobank, but the analytic linearization formulas for the model have not')
    disp('been adjusted. if you want to run with this value of rhobank, set repl=1 and the model will ')
    error('be solved by numerical linearization')
end

givenames;

if irun == 1
    solvemodel

    simulate

elseif irun == 2

    fdiff=1;
    irun=1;
    solvemodel
    getVAR

elseif irun == 0 || irun == 3 || irun ==4 || irun ==5 || irun==6 || irun==7 || irun==8

    set_params_priors;
    setup;
    do;
    xx=imap(xx);

    loaddata

    if Taylor > 3
        error('fatal (main) simulation code not ready for Monetary Base Rule')
    end

    if irun ==0 || irun==7 || irun == 8

        %irun=1;
        if entr == 2;params(1)=1; elseif entr==-4; params(1)=-3; end
        iirun=irun;
        if irun == 8;iirun=0;end
        [s_alphas,s_alpha_2,s_alpha_1,s_alpha_0,s_betas,s_beta_0,s_beta_1,taumat,M,ssvalues,ttt2,G,Gp,Gs] = ...
            makealphabeta(lambdaz,g,her,ksi, pi,i,omegabar,Rk,n,ev,mb,uzc,Ra,c,w,Re,m3, ...
            k,params,q,s,u,dm,vn,RT,Rm,icheck,Taylor,forward,parstoch, ...
            etai,taushock,iirun,btot,phi,y,R,rk,l,kbar);
        if entr == 2;params(1)=2; elseif entr==-4;params(1)=-4; end

        size_a=size(s_alpha_0);
        size_b=size(s_beta_0);

        if irun == 0 || irun==7

            maxlike

        elseif irun == 8
            if country == 1
                xxnew=[0.747009281292349   0.026985805494254  27.730414991452072  39.148879678134747, ...
                    0.719653763134011   1.823718023059353   0.066407338681933   0.871581131612255, ...
                    0.923290086242506   0.614536593687414   0.916940911185091   0.255920780898203, ...
                    0.251067695707317   0.979586845470754   0.978876991776108   0.977945641996660, ...
                    0.981903340486590   0.988777949527175   0.071307075366927   0.872305427887599, ...
                    0.971993739747061   0.958625086067571   0.933322715030591   0.599359557073978, ...
                    0.953361382118239   0.935767518333359   0.009130214496604   0.090992882936806, ...
                    0.003325792432916   0.026317310123598   0.012164332406474   0.004540644954519, ...
                    0.003608633564210   0.001137723701032   0.003917908252165   0.453423772790887, ...
                    0.045915700845334   0.028093214188939   0.016041546701119   0.017972397538813, ...
                    0.150290879579772   0.002991178327047   0.014422038736037   0.000646775478460, ...
                    0.000982028128691   0.000468251831614   0.000097114228687   0.000132207720523, ...
                    0.000259255220832];
            elseif country == 2
                xxnew = [0.694873177330786  20.388717209234240  29.223148937142053   0.669813803908465 ...
                    1.852573874712145   0.878760889724742   0.327661906640329   0.638218651150825 ...
                    0.935003055176190   0.202503436340635   0.319653705082079   0.723505892535903 ...
                    0.492089671159898   0.985630026574753   0.982755958505151   0.978891059508739 ...
                    0.945464130132968   0.177895620833271   0.561477315668083   0.938356042773474 ...
                    0.850857465173620   0.903993421844338   0.443207452482997   0.951322235158157 ...
                    0.873494719566081   0.014713123590438   0.007081605294395   0.079415391402219 ...
                    0.003347465236391   0.018340080855256   0.020746748428549   0.006693790457139 ...
                    0.004030862918302   0.000270357075958   0.004635377429835   0.526391676331539 ...
                    0.119044805749658   0.052289805523720   0.019303307014312   0.018354932774679 ...
                    0.132420689898704   0.005777866040847   0.016549684493162   0.000904763875621 ...
                    0.001551923184484   0.000708307313703   0.000047809909475   0.000262389375309 ...
                    0.000250800210888   0.000681568649884];
            end
            xx=imap(xxnew);
            [floglik,cost_flag,H,F,Q,initV,is,alf,RR,V_eps]   =  ssloglik(xx,par,taumat,her,Rb,Re,c,y,i,mb, w,l,k,n, pi,g,ev, ...
                taushock,Pe,G,Gp,omegabar,Rk,rk,Gs,fdiff,Xdat,J,ric,M,m1,currdep,Ra,dm,m3, ...
                demean,mm,parstoch,duration1,tauvar,tauvar2,size_a,size_b,prior,taushock1,gamma,alpha, ...
                R,ttt2,entr,taushock3,duration2,params,q,s,u,lambdaz,uzc,vn,Rm,btot,init_kalman);

            if country == 2
                if abs(-4.801125164052886e+003-floglik) > .1e-7
                    error('fatal (main) something wrong with parameter values')
                end
            elseif country == 1
                if abs(-5.160902520725555e+003-floglik) > .1e-7
                    error('fatal (main) something wrong with parameter values')
                end
            end
            lambda_hp = 1600;
            redo = 1;%=1, compute model second moments and store for use by maketablesfacts
            if redo==1
                get2moments(lambda_hp,country,xxnew,par,taumat,her,Rb,Re,c,y,i,mb, w,l,k,n, pi,g,ev, ...
                    taushock,Pe,G,Gp,omegabar,Rk,rk,Gs,fdiff,Xdat,J,ric,M,m1,currdep,Ra,dm,m3, ...
                    demean,mm,parstoch,duration1,tauvar,tauvar2,size_a,size_b,prior,taushock1,gamma,alpha, ...
                    R,ttt2,entr,taushock3,duration2,params,q,s,u,lambdaz,uzc,vn,Rm,btot,init_kalman,outfolder);
            end

            hp_filter = 1; %=1 hp-filtered data; otherwise, first differenced.
            maketablesfacts(hp_filter,outfolder,country,lambda_hp,curdrive,namedir)
        end

    elseif irun == 3
        nT=find(date==2003.25); %select the last data point you want to include in the estimation. The forecast will start in nT+1

        nlags=2;%number of lags to be used in the VAR
        hasconstant=1; %=1, you want to include the constant
        [Yhat] = varforecast(nT,nlags,Xdat,hasconstant);
        T=size(Xdat,2);
        tt=nlags+1:max(nT+8,T);
        tt1=nT+1:tt(end);
        plotvarforecast

    elseif irun ==4 || irun ==5

        %note that we need to load all parameters, not only xx if we have estimated steady-state parameters. The reason is that we need to go through unld to recompute the steady state at the optimised parameters
        if country == 1
            if entr == -2
                load results_cee_ea xx par
            elseif entr == 1
                load results_cmr_ea xx par
            elseif entr == -3
                load results_bgg_ea xx par
            elseif entr ==2
                load results_cmr_Fisher_ea xx par
            end
        elseif country == 2
            if entr == -3
                load results_bgg_us xx par
            elseif entr == 1
                load results_cmr_us xx par
            elseif entr==-2
                load results_cee_us xx par
            end
        end
        xx=imap(xx);
        undo; loadparameters;
        
        iirun=1;
        [s_alphas,s_alpha_2,s_alpha_1,s_alpha_0,s_betas,s_beta_0,s_beta_1,taumat,M,ssvalues,ttt2,G,Gp,Gs] = ...
            makealphabeta(lambdaz,g,her,ksi, pi,i,omegabar,Rk,n,ev,mb,uzc,Ra,c,w,Re,m3, ...
            k,params,q,s,u,dm,vn,RT,Rm,icheck,Taylor,forward,parstoch, ...
            etai,taushock,iirun,btot,phi,y,R,rk,l,kbar);
        size_a=size(s_alpha_0);
        size_b=size(s_beta_0);
        
        [alphas,alpha_2,alpha_1,alpha_0,betas,D,rho,V_eps,Vmeas,l_psi,l_x,l_eps, ...
            tau,x,gamma,alpha,tauc,taud,beta,b,upsil,ksi,psil,psik,muzstar,Taylor,params,Re,c,y,i,w,l, ...
            rk,k,n,pi,g,Pe,G,Gp,omegabar,Rk,Gs,sigma,her,Rb,mb,m,ev,m1,currdep,Ra,R,dm,m3,q,s,u,lambdaz,uzc,vn,Rm,btot,M] = ...
            unld(xx,par,taushock,tauvar,tauvar2,parstoch,XXX,taushock1,taushock3,size_a,size_b,params, ...
            Re,c,y,i,w,l,k,n,pi,g,Pe,G,Gp,omegabar,Rk,rk,Gs,her,Rb,mb,ev,m1,currdep,Ra,R,dm,m3,q,s,u,lambdaz,uzc,vn,Rm,btot,M);

        if entr == 2;params(1)=1; elseif entr==-4; params(1)=-3; end
        [alphas,alpha_2,alpha_1,alpha_0,betas,beta_0,beta_1,taumat,M,ssvalues,ttt2,G,Gp,Gs] = ...
            makealphabeta(lambdaz,g,her,ksi, pi,i,omegabar,Rk,n,ev,mb,uzc,Ra,c,w,Re,m3, ...
            k,params,q,s,u,dm,vn,RT,Rm,icheck,Taylor,forward,parstoch, ...
            etai,taushock,1,btot,phi,y,R,rk,l,kbar);
        if entr == 2;params(1)=2; elseif entr==-4;params(1)=-4; end
        size_a=size(alpha_0);
        size_b=size(beta_0);

        [initxi,F,H,Q,RR,rho,D,A,B1,V_eps,initV,l_z,l_psi,is,alf,taum,taub,taus,tausb,alpha,b,beta, pi,Re,Ra, ...
            muzstar,upsil,R] = prepforecast(accuracy,xx,par, ...
            taumat,her,Rb,Re,c,y,i,mb,w,l,k,n, pi,g,ev,taushock,Pe,G,Gp,omegabar,Rk,rk,Gs,fdiff,Xdat,J,ric,M, ...
            m1,currdep,Ra,dm,m3,demean,mm,parstoch,duration1,tauvar,tauvar2,size_a,size_b,taushock1,R,ttt2,entr,taushock3, ...
            duration2,params,q,s,u,lambdaz,uzc,vn,Rm,btot);

        if irun ==5
            init_sim    = find(date==start_estimation); %start the simulation at period init_sim; if init_sim>1, the decomposition is based on separating out (from a given quarter, init_sim, onwards): (a) the unwinding of initial state and (b) the contribution of new innovations occurring from init_sim and their unwinding. So the initial state is considered init_sim-1
            start_plot  = find(date==1987.0);   %plots will show data starting in start_plot
            vert_bar    = []; %e.g. =2006.25 if you want to add a vertical bar at a specified date in the plot; [] otherwise. The bar is set between the period you specified and the following one

            plot_levels = 2; %=0, plot variables as they are; =1, plot levels; =2 plot year-on-year growth rates

            cell_datavintage = {'ea_March09_final', 'present',  1;  %1st column specifies the name of the file with the data
                'Dec08_final',      'past',     0}; %2nd column specifies which is the present round and which is the past round
            %in 3rd column set =1 to specify the vintage you want to run; the other entries in 3rd column have to be zero

            show_initialstate = 0; %=1, chart with bars will show a bar with contribution of initial state; =0, chart does not show contribution of initial state

            categ = {%'Inflation Objective',      [2],                 [0 0 1]; %1th column: name of categories of shocks
                'Aggregate Demand',                   [10 11 13 15 20 22], [0 1 0]; %[0 1 0] %2nd column: shocks pertaining to each category
                %'Banking',                 [3 4 5 6],           [0 1 1]; %3rd column: colour of each category you want to use in the plot
                'Labour-specific Productivity Factors',   [16 18],    [1 1 0]; %[1 1 0]
                'Markups',                  [1 12 24 ],          [0 1 1]; %[0 1 1]
                'Monetary Policy',                   [2 19],               [1 0 0] ; %[1 0 0]
                %'Other shocks' [1  10 11 12 13 15 16 18 20 22 24], [1 0 0];
                %'Financial Factors',        [7 8 9 14 17 21 23],    [0.83 0.82 0.78];
                %'Liquidity Preferences',    [6 9], [.83 0.82 0.78]};
                'Money Demand', [3 4 5 6 8 9 14],         [1 0.7 0];
                'Capital Formation',        [7 17 21 23 25],       [.83 0.82 0.78]};
            %'Term Premium',             [ 25],                [0.7 0.2 0]}; %[0.7 0.2 0]};


            [categ_,cmap,signal_on,Corr_sample,Relvar_sample] = shockanalysis(Xdat,F,H,Q,RR,initxi,initV,l_z,l_psi,rho,A,B1,J,init_sim,shockss,taushock, ...
                parstoch,date,tauvar,is,alf,taum,taub,taus,tausb,mm,trendm3,trendm1,fdiff, ...
                plot_levels,outfolder,entr,start_plot,categ,vert_bar,alpha,b,beta,start_estimation,init_kalman,end_estimation,user,cell_datavintage,show_initialstate);

            %compute variance decomposition
            index_level = [];  %location of variables (in full list not tauvar) you want to transform into levels; Empty if you want to leave all variables as they are in Xdat
            if show_initialstate ==1
                cmap = cmap(1:end-1,:);
            end
            vardecomp(H,F,Q,V_eps,RR,B1,D,is,index_level,fdiff,tauvar,taushock,categ_,cmap,signal_on,Corr_sample, ...
                Relvar_sample,outfolder,entr,cdir,Xdat(:,find(date==start_estimation)+init_kalman:find(date==end_estimation)),ric,l_z,l_psi,parstoch);


            plot_figure =0; %=1 plot decomposition of stock market, inflation and gdp for euro area and US on the same chart
            if plot_figure ==1
                load([cdir '\results_ea\contributions'])

                contrib_stock_ea = contrib_stock;
                contrib_infl_ea  = contrib_infl;
                contrib_gdp_ea   = contrib_gdp;
                load([cdir '\results_us\contributions'])

                figure, orient landscape
                for i=1:5
                    subplot(5,2,(i-1)*2+1)
                    plot(contrib_gdp_ea(1).date,contrib_gdp_ea(1).fitted,'k-',contrib_gdp_ea(1).date,contrib_gdp_ea(i).value,'r.-')
                    title(contrib_gdp_ea(i).name)
                    axis tight

                    subplot(5,2,2*i)
                    plot(contrib_gdp(1).date,contrib_gdp(1).fitted,'k-',contrib_gdp(1).date,contrib_gdp(i).value,'r.-')
                    title(contrib_gdp(i).name)
                    axis tight
                end
                str = 'Figure 8. Smoothed GDP (-) and Data driven by just the indicated shock (*). EA (left) and US (right)';
                hout = suptitle(str);

                figure, orient landscape
                for i=1:5
                    subplot(5,2,(i-1)*2+1)
                    plot(contrib_infl_ea(1).date,contrib_infl_ea(1).fitted,'k-',contrib_infl_ea(1).date,contrib_infl_ea(i).value,'r.-')
                    title(contrib_infl_ea(i).name)
                    axis tight

                    subplot(5,2,2*i)
                    plot(contrib_infl(1).date,contrib_infl(1).fitted,'k-',contrib_infl(1).date,contrib_infl(i).value,'r.-')
                    title(contrib_infl(i).name)
                    axis tight
                end
                str = 'Figure 9. Smoothed Inflation (-) and Data driven by just the indicated shock (*). EA (left) and US (right)';
                hout = suptitle(str);

                if entr==1
                    figure, orient landscape
                    for i=1:5
                        subplot(5,2,(i-1)*2+1)
                        plot(contrib_stock_ea(1).date,contrib_stock_ea(1).fitted,'k-',contrib_stock_ea(1).date,contrib_stock_ea(i).value,'r.-')
                        title(contrib_stock_ea(i).name)
                        axis tight

                        subplot(5,2,2*i)
                        plot(contrib_stock(1).date,contrib_stock(1).fitted,'k-',contrib_stock(1).date,contrib_stock(i).value,'r.-')
                        title(contrib_stock(i).name)
                        axis tight
                    end
                    str = 'Figure 10. Smoothed Stock market (-) and Data driven by just the indicated shock (*). EA (left) and US (right)';
                    hout = suptitle(str);
                end
            end


        elseif irun ==4
            nT = find(date==2007.75); %2001.5; %2008.25 %select the last data point included in the sample used for estimation. The forecast will start in nT+1
            start_plot = find(date==2008.75); %2007.0; %specify the first data point you want to display in the chart

            plot_levels = 2; %=0, plot ratios of variables; =1, plot levels; =2 plot year-on-year growth rates

            ecb_projection = 1; %=1 variable are expressed in %, some are annualised, the mean is added; =0 no adjustment is made to the data

            if nT == length(date)
                LL=12; %number of quarters for which you want to forecast ahead
            else
                LL=length(date)-nT; %LL=10;
            end

            [Yhat,Ydata,VYhat] = forecastkalman1(Xdat,H,F,Q,RR,B1,initV,nT,LL,alf,initxi,tauvar,plot_levels,meanxxx,mm, ...
                trendm3,trendm1,fdiff,alpha,b,beta,is,taum,taub,taus,tausb,l_z,l_psi,D,taushock,outfolder,parstoch,date,shockss,user,V_eps);

            plotkalmanforecast(Xdat,Yhat,VYhat,date,LL,nT,tauvar,fdiff,pi,Re,Ra,muzstar,upsil,ecb_projection,start_plot,meanxxx,demean,alf,R,entr);

            %decompose forecast into dynamics coming from endogenous state and exogenous state
            decomp_forecast(Xdat,F,H,Q,RR,initxi,initV,l_z,l_psi,rho,A,B1,date,alf,taum,taub,taus,tausb,mm,trendm3,trendm1,tauvar,is,J,fdiff,nT,LL,parstoch,taushock,user)

            rec_forecast=0; %=1, compute model performance in recursive out-of-sample exercise
            if rec_forecast==1
                init_rec = 2001.5; %last data point that will be included in estimation sample before starting out-of-sample forecast
                end_rec  = 2008.25; %data point you want to end the out-of-sample exercise (the program will automatically extend the sample recursively up to end_rec-1 to leave at least one observation for validation)
                LL = 12; %number of quarters you want to forecast ahead
                yperiods = 2; %=1 if model is re-estimated each quarter; =2 if every 2 quarter; =3 if every 3 quarters; =4 if every 4 quarters
                r_mm = [ 15];%variables that needs to be demeaned also if demean=0 (numbering refers to full list of variables, not tauvar)
                opt_routine =4; %routine to be used in recursive estimation

                if entr==1
                    if country ==1
                        index_r = [1 1 0 1 1 1 1 1 1 1 0 0 0 0 1 1]; % set 1 for variables that are in quarterly growth rate (and you want to display RMSE in terms of y-o-y) and 0 for variables in level (e.g. interest rate)
                    elseif country ==2
                        index_r = [1 1 0 1 1 1 1 1 1 1 0 0 0 0 1 1 1]; % set 1 for variables that are in quarterly growth rate (and you want to display RMSE in terms of y-o-y) and 0 for variables in level (e.g. interest rate)
                    end
                    taufor = [1:size(Xdat,1)]; %specify which of the variables in Xdat will be included in the computation of the multivariate measure of forecasting accuracy. If you want to include all variables, set taufor=1:size(Xdat,1). If you want to include a subset only, specify the relevant number of row of Xdat (e.g. taufor=[2 5 13] if you want Dp, Dy and Re)
                elseif entr==-2
                    index_r = [1 0 1 1 1 1 0 1 1]; % set 1 for variables that are in quarterly growth rate (and you want to display RMSE in terms of y-o-y) and 0 for variables in level (e.g. interest rate)
                    taufor = [1:size(Xdat,1)]; %specify which of the variables in Xdat will be included in the computation of the multivariate measure of forecasting accuracy. If you want to include all variables, set taufor=1:size(Xdat,1). If you want to include a subset only, specify the relevant number of row of Xdat (e.g. taufor=[2 5 13] if you want Dp, Dy and Re)
                elseif entr==-3
                    index_r = [1 1 0 1 1 1 1 1 0 0 1 1]; % set 1 for variables that are in quarterly growth rate (and you want to display RMSE in terms of y-o-y) and 0 for variables in level (e.g. interest rate)
                    taufor = [1:size(Xdat,1)]; %specify which of the variables in Xdat will be included in the computation of the multivariate measure of forecasting accuracy. If you want to include all variables, set taufor=1:size(Xdat,1). If you want to include a subset only, specify the relevant number of row of Xdat (e.g. taufor=[2 5 13] if you want Dp, Dy and Re)
                end

                forecast_rec
            end

        elseif irun ==6
            error('fatal (main) you have selected irun=6 but the code is not ready for this')
        end
    end

end
