function  micro(sigma_SS,omegabar,Rk,Re,k,n,Pe_SS,mu)

ick=0;%controls whether it is sigma (ick=0) or sp1 (ick=1) that is perturbed
indif=1;%if unity, then compute indifference curves (always do this)
om=1;% if infif = 1, om=1 means do the indifference curve in (omega,leverage), om=0 means do it in (Pe,leverage) space
   %warning, om = 0 is very time-intensive!



%sp1 and sig are exogenous in these calculations
sp1=(1+Rk)/(1+Re);
sig=sigma_SS;

lev_SS=k/n;
Pe_SS=sp1*omegabar*lev_SS/(lev_SS-1);

%take sp1 and sig as given and find Pe and lev that solve the CSV
%contracting problem

%step 1: find omega that solves the first order condition
[ff,G,F,Gamma,Gam_muG,Fprime]=findomeg(omegabar,sig,mu,sp1);
if abs(ff) > .1e-10
    error('fatal(micro) omegabar not a steady state')
end
%step 2: leverage and spread
lev=1/(1-Gam_muG*sp1);
Pe=sp1*omegabar*lev/(lev-1);
%the level of utility at the steady state equilibrium
Ubar=(1-Gamma)*sp1*lev;

if abs(lev - lev_SS) > .1e-10 | abs(Pe-Pe_SS) > .1e-10 
    error('fatal (micro) not a steady state')
end

%plot utility as a function of (i) omega and leverage, and (ii) as a function of Pe
%and leverage

%&&&&&&& for my handout &&&&&&&&&&&&
% handout graph of utility as a function of leverage
lever=[];
util=[];
utilp=[];
lever=1.001:.01:6;
for ii = 1:length(lever)
    leve=lever(ii);
    [ff,omega,z,G,F,Gamma,er] = leverage(Pe,leve,sp1,sig,Ubar);
    util(ii)=(1-Gamma)*sp1*leve;
    [ff,omega,z,G,F,Gamma,er] = leverage(1.5,leve,sp1,sig,Ubar);
    utill(ii)=(1-Gamma)*sp1*leve;
    [ff,omega,z,G,F,Gamma,er] = leverage(Pe,leve,sp1,sig*1.02,Ubar);
    utilp(ii)=(1-Gamma)*sp1*leve;
end
plot(lever,util,lever,utill,'x-',lev_SS,Ubar,'*')
ss=['Z/(1+R) = ',num2str(Pe_SS)];
ss1=['Z/(1+R) = ',num2str(1.5)];
legend(ss,ss1)
ylabel('Expected return for entrepreneur','fontsize',32)
xlabel('leverage','fontsize',32)
title('Expected entrepreneurial return, over opportunity cost, N(1+R)','fontsize',38)
axis tight


%&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
% do (i) first
oomega=.001:.01:5;
for ii=1:length(oomega)
    [G,F,Gamma] = valuation(oomega(ii),sig);
    Util(ii)=(1-Gamma)*sp1*lev_SS;
end

[Y,I]=min(abs(oomega-omegabar));

figure('Name','entrepreneurial utility function')
subplot(221)
plot(oomega,Util,omegabar,Ubar,'*')
ylabel('entrepreneurial utility')
xlabel('omega_bar')
title('utility(omega,lev) as a function of omega')
axis tight
L=1.1:.001:20;
subplot(222)
plot(L,Ubar*(L/lev),lev_SS,Ubar,'*')
ylabel('entrepreneurial utility')
xlabel('leverage')
title('utility(omega,lev) as a function of lev')
axis tight

% now go for (ii)
%display the level of entrepreneurial utility as a function of leverage

lever=[];
util=[];
utilp=[];
lever=1.001:.01:6;
for ii = 1:length(lever)
    leve=lever(ii);
    [ff,omega,z,G,F,Gamma,er] = leverage(Pe,leve,sp1,sig,Ubar);
    util(ii)=(1-Gamma)*sp1*leve;
    [ff,omega,z,G,F,Gamma,er] = leverage(1.5,leve,sp1,sig,Ubar);
    utill(ii)=(1-Gamma)*sp1*leve;
    [ff,omega,z,G,F,Gamma,er] = leverage(Pe,leve,sp1,sig*1.02,Ubar);
    utilp(ii)=(1-Gamma)*sp1*leve;
end
%plot(lever,util,lever,utilp,'*-')
%legend('baseline','perturbed')
subplot(223)
plot(lever,util,lever,utill,'x-',lev_SS,Ubar,'*')
ss=['Z/(1+R) = ',num2str(Pe_SS)];
ss1=[', Z/(1+R) = ',num2str(1.5)];
legend([ss,ss1])
ylabel('entrepreneurial utility')
xlabel('leverage')
title('utility(Pe,lev) as a function of lev')
axis tight

Util=[];
PPe=1.001:.001:Pe_SS*(1+.2);
for ii=1:length(PPe)
    omega=(PPe(ii)/sp1)*(lev_SS-1)/lev_SS;
    [G,F,Gamma] = valuation(omega,sig);
    Util(ii)=(1-Gamma)*sp1*lev_SS;
end

subplot(224)
plot(PPe,Util,Pe_SS,Ubar,'*')
ylabel('entrepreneurial utility')
xlabel('Pe')
title('utility(Pe,lev) as a function of Pe')
axis tight
%&&&&&&&&&&&&& end of utility function plot &&&&&&&&&&&&&&&

%now, go for the indifference curve...

llev=1.001:.01:1.2;
PPe=1.001:.001:Pe*(1+.2);
UU=Ubar;
if indif == 1;
    UU=Ubar*[.7:.1:3];llev=1.001:.01:6;PPe=1.001:.001:Pe*(1+.4);
end

%easy indifference curve, the one in the (omega, leverage) space:
if indif == 1 & om == 1
    
    ttt=[];
    oomega=[];
    figure('Name','indifference curve, U(omega,lev)')
    
    oomega=.001:.001:.75;
    for ii=1:length(oomega)
        [G,F,Gamma] = valuation(oomega(ii),sig);
        ttt(ii)=(1-Gamma);
    end

    [Y,I]=min(abs(UU-Ubar));
    [Y1,I1]=min(abs(oomega-omegabar));
    
    subplot(211)

    for jj = 1:length(UU)
        Ljj=UU(jj)./(ttt*sp1);
        %only interested in positive borrowing case, i.e., leverage>=1
        JJ=find(Ljj>=1);
        plot(Ljj(JJ),oomega(JJ))
        hold on

        plot(lev_SS,omegabar,'*')

    end
    
    ylabel('\omega - bar')
    xlabel('leverage')
    title('Indifference curve, (leverage, \omega - bar) space')
    axis tight
    
    tt2=[];
    oomega=.001:.01:15;
    for ii=1:length(oomega)
        [G,F,Gamma] = valuation(oomega(ii),sig);
        tt2(ii)=1-(Gamma-mu*G)*sp1;
    end

    subplot(212)
    plot(1./tt2,oomega,lev_SS,omegabar,'*')
    ylabel('\omega - bar','fontsize',36)
    xlabel('leverage','fontsize',36)
    title('Bank zero profit condition, in (leverage, \omega - bar) space','fontsize',46)
    axis tight

end

if indif == 1 & om ~= 1
    %now go for the indifference curves in the (leverage, Pe) space
    %careful...this graph takes a long time to compute!
    figure('Name','Indifference curve, (Pe,leverage) space')
    Iind=[];
    UU=Ubar*[.8:.01:1.2];llev=1.001:.01:6;PPe=1.001:.001:Pe*(1+.4);
    for ii = 1:length(UU)
        U=UU(ii);
        [indPe] = indifference(llev,PPe,sp1,sig,U);Iind=find(indPe>.0001);
        if length(UU) > 1
            plot(llev(Iind),indPe(Iind))
            hold on
        end
    end
    plot(lev_SS,Pe_SS,'*')
    ylabel('Z/(1+R), risk spread')
    xlabel('leverage, Assets/Net Worth')
    title('Entrepreneurial indifference curves')
    axis tight
end

%now go for the comparative static exercise....in the (Pe,leverage) space
%first, get the zero profit function and indifference curves at the steady
%state equilibrium
llev=1.001:.01:1.2;
PPe=1.001:.001:Pe*(1+.2);
[indPe] = indifference(llev,PPe,sp1,sig,Ubar);
[profPe] = zprofits(llev,PPe,sp1,sig,mu);
Iind=find(indPe>.0001);
Iprof=find(profPe>.0001);


%now go for the pertubed equilibrium

if ick==0 
    llevt=llev;
    ip=1.05;
    ss=['Effect of a ',num2str(100*(ip-1)),'% jump in \sigma'];
    [omega,er] = ffindomeg(sig*ip,mu,sp1);
end
if ick==1;
    llevt=1.17:.01:1.25;
    ip=1.02;
    ss=['effect of a ',num2str(100*(ip-1)),'% jump in s'''];
    sp1=sp1*ip;
    ip=1;
    [omega,er] = ffindomeg(sig,mu,sp1);
end

if er == 1
    error('fatal (micro) failed to find omega')
end
[ff,G,F,Gamma,Gam_muG,Fprime]=findomeg(omega,sig*ip,mu,sp1);
if abs(ff) > .1e-10
    error('fatal (micro) mistake')
end
lev=1/(1-Gam_muG*sp1);
Pe=sp1*omega*lev/(lev-1);
Ubart=(1-Gamma)*sp1*lev;

[indPet] = indifference(llevt,PPe,sp1,sig*ip,Ubart);
[profPet] = zprofits(llevt,PPe,sp1,sig*ip,mu);
Iindt=find(indPet>.0001);
Iproft=find(profPet>.0001);

figure('name','comparative static exercise')
plot(llev(Iind),400*(indPe(Iind)-1),llev(Iprof),400*(profPe(Iprof)-1),llevt(Iindt),400*(indPet(Iindt)-1),llevt(Iproft),400*(profPet(Iproft)-1),lev_SS,400*(Pe_SS-1),'*',lev,400*(Pe-1),'*')
xlabel('leverage, qK/N','fontsize',28)
ylabel('risk spread (APR)','fontsize',20)
title(ss,'fontsize',36)
%axis([1.1 1.14 1.5 4])
axis tight

figure('Name','static equilibrium (lev,omega) space')
tt2=[];
ttt=[];
tttt=[];
ttt2=[];
dd=2;

oomega=.000001:.001:.3;
for ii=1:length(oomega)
    [G,F,Gamma] = valuation(oomega(ii),sig);
    tt2(ii)=1-(Gamma-mu*G)*sp1;
    ttt(ii)=(1-Gamma);
    [G,F,Gamma] = valuation(oomega(ii),sig*dd);
    ttt2(ii)=1-(Gamma-mu*G)*sp1;
    tttt(ii)=(1-Gamma);
end
%leverage implied by utility function, for given Ubar and omega
Ljj=Ubar./(ttt*sp1);
%only interested in positive borrowing case, i.e., leverage>=1
JJ=find(Ljj>=1);
plot(1./tt2,oomega,'+-',Ljj(JJ),oomega(JJ),lev_SS,omegabar,'*')
hold on

[omega,er] = ffindomeg(sig*dd,mu,sp1);
[ff,G,F,Gamma,Gam_muG,Fprime]=findomeg(omega,sig*dd,mu,sp1);
if abs(ff) > .1e-10 | isnan(ff) == 1
    error('fatal (micro) mistake')
end
lev=1/(1-Gam_muG*sp1);
Pe=sp1*omega*lev/(lev-1);
Ubart=(1-Gamma)*sp1*lev;
LLjj=Ubart./(tttt*sp1);
JJJ=find(LLjj>=1);
plot(1./ttt2,oomega,'+-',LLjj(JJJ),oomega(JJJ),lev,omega,'o')

ylabel('\omega - bar','fontsize',36)
xlabel('leverage','fontsize',36)
title('Static equilibrium','fontsize',46)
axis tight

oomega=.001:.01:5;
f=[];
fp=[];
for ii = 1:length(oomega)
    omega=oomega(ii);
    z       =   (log(omega)+sig^2/2)/sig;
    f(ii)       =   normpdf(z);
    zp       =   (log(omega)+(sig*dd)^2/2)/(sig*dd);
    fp(ii)       =   normpdf(zp);
end

plot(oomega,f,oomega,fp)
ylabel('density')
xlabel('/omega')
title('Impact on log normal density of doubling standard deviation')
axis tight
